#ifndef POKEPLATINUM_UNK_020067E8_H
#define POKEPLATINUM_UNK_020067E8_H

#include "struct_decls/struct_020067E8_decl.h"
#include "struct_defs/struct_0208BE5C.h"

UnkStruct_020067E8 * sub_020067E8(const UnkStruct_0208BE5C * param0, void * param1, const int param2);
void sub_02006814(UnkStruct_020067E8 * param0);
void * sub_0200681C(UnkStruct_020067E8 * param0, int param1, int param2);
void * sub_0200682C(UnkStruct_020067E8 * param0);
void sub_02006830(UnkStruct_020067E8 * param0);
void * sub_02006840(UnkStruct_020067E8 * param0);
BOOL sub_02006844(UnkStruct_020067E8 * param0);

#endif // POKEPLATINUM_UNK_020067E8_H
